package J48final;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import weka.classifiers.trees.J48;
import weka.core.Instances;
import weka.core.converters.ConverterUtils.DataSource;

public class DecisionTree {

	public static void main_J48_ML(String[] args) {
		String trainFile = "E:/trainsetML.csv";
		String testFile = "E:/testset.csv";
		float CF = 0.1f;
		int MinNumObj = 20;
		
		try {
			writeToFile("E:/Result/", "testResultJ48_ML.csv", J48_ML(trainFile,testFile,CF,MinNumObj).toString().getBytes());
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public static void main(String[] args) {
		String trainFile = "E:/trainsetMC.csv";
		String testFile = "E:/testset.csv";
		float CF = 0.015f;
		int MinNumObj = 24;

		try {
			writeToFile("E:/Result/", "testResultJ48_MC.csv", J48_MC(trainFile,testFile,CF,MinNumObj).toString().getBytes());
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * @param trainFile, each line is a project instance with 12 project indicators and 39 labels.
	 * @param testFile, each line is a project instance with 12 project indicators.
	 * @param CF,the confidence factor CF. 
	 * @param minNumObj,the minimum number of objects MinNumObj.
	 * @return The prediction results: testResult, each line is a project instance with different predictions.
	 */
	public static StringBuffer J48_ML(String trainFile,String testFile,float CF,int minNumObj ){
		return trainAndTest(trainFile,testFile,CF,minNumObj,39);
	}
	
	/**
	 * @param trainFile, each line is a project instance with 12 project indicators and newly defined classes.
	 * @param testFile, each line is a project instance with 12 project indicators.
	 * @param CF,the confidence factor CF. 
	 * @param minNumObj,the minimum number of objects MinNumObj.
	 * @return The prediction results: testResult, each line is a project instance with the prediction (newly defined classes).
	 */
	public static StringBuffer J48_MC(String trainFile,String testFile,float CF,int minNumObj){
		return trainAndTest(trainFile,testFile,CF,minNumObj,1);
	}

	/**
	 * 
	 * @param trainFile, each line is a project instance with 12 project indicators and at least one labels.
	 * @param testFile, each line is a project instance with 12 project indicators.
	 * @param CF,the confidence factor CF. 
	 * @param minNumObj,the minimum number of objects MinNumObj.
	 * @param classNum,the class number Of trainFile
	 * @return The prediction results: testResult, each line is a project instance with different predictions.
	 */
	public static StringBuffer trainAndTest(String trainFile,String testFile,float CF,int minNumObj,int classNum){
		StringBuffer preResult = new StringBuffer();
		try{
			Instances instance1=DataSource.read(trainFile);
			Instances train;
	
			Instances testData=DataSource.read(testFile);
			
			//Keep the testFile
			String[] preSourceData = new String(readFile(testFile),"GBK").split("\r\n");
			
			//keep the predictions for all labels
			for(int i=0;i<classNum;i++){
				train = new Instances(instance1);
				int count=0;
				for(int j=0;j<classNum;j++){
					if(j!=i){
						train.deleteAttributeAt(count+12);
					}else{
						count=count+1;
					}
				}
				
				//The number of features
				train.setClassIndex(12);

				Instances test = new Instances(testData);
				//Add the empty column to add the prediction
				test.insertAttributeAt(train.attribute(train.numAttributes() - 1), test.numAttributes());
				test.setClassIndex(test.numAttributes()-1);

				J48 j48=new J48();
				j48.setMinNumObj(minNumObj);
				j48.setNumFolds(10);
				j48.setSeed(1);
				j48.setConfidenceFactor(CF);
				j48.setReducedErrorPruning(false);
//				j48.setDebug(true);
				j48.buildClassifier(train);
				
				
				preSourceData[0]=preSourceData[0]+","+train.classAttribute().name();
				for(int k=0;k<test.numInstances();k++){
					double d = j48.classifyInstance(test.instance(k));
					String prediction=test.classAttribute().value((int)d);
					preSourceData[k+1]=preSourceData[k+1]+","+prediction;
				}
			}
			
			for(int i=0;i<preSourceData.length;i++){
				preResult.append(preSourceData[i]).append("\r\n");
			}
		}catch(Exception e){
			e.printStackTrace();
		}
		
		return preResult;
	}
	
	/**
	 * reafFile
	 * @param fileName 
	 * @return 
	 * @throws Exception
	 */
	public static byte[] readFile(String fileName) throws Exception{
		File file = new File(fileName);
		if(file.exists()){
			FileInputStream fis = null;
			ByteArrayOutputStream bos = null;
			try {
				fis = new FileInputStream(fileName);
				bos = new ByteArrayOutputStream(fis.available());
				byte[] buffer = new byte[1024];
				int count = 0;
				while ((count = fis.read(buffer)) != -1) {
					bos.write(buffer, 0, count);
				}
				bos.flush();
				return bos.toByteArray();
			}catch(Exception e){
				throw new Exception("read file failed!!",e);
			}finally{
				if(bos!=null){
					try {
						bos.close();
					} catch (IOException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
				if(fis!=null){
					try {
						fis.close();
					} catch (IOException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
			}
		}else{
			throw new Exception(fileName+" file doesn't exist!!pass it !!");
		}

	}
	
	/**
	 * writeToFile
	 * @param path 
	 * @param fileName 
	 * @param fileEncode 
	 * @param content 
	 * @throws TranFailException
	 */
	public static void writeToFile(String path,String fileName,byte[] content) throws Exception{
		File file = new File(path);
		if(file.exists()){
			if(file.isFile()){
				throw new Exception(path+" is a file,pass!!");
			}
		}else{
			file.mkdirs();
		}
		file = new File(path+"/"+fileName);
		try {
			file.createNewFile();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			throw new Exception(path+"/"+fileName+" faild to create!!!", e);
		}
		
		FileOutputStream fos = null;
		try {
			fos = new FileOutputStream(path+"/"+fileName);
   			fos.write(content);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			throw new Exception( path+"/"+fileName+" failed to write!!", e);
		}finally{
			if(fos!=null){
				try {
					fos.flush();
					fos.close();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	}

}
